/*
 * pwm.c
 *
 *  Created on: 2014-1-20
 *      Author: a0220211
 */

#include "pwm.h"

//=========================================================================
// het interrupt enable function
//=========================================================================
void het_int_enable(void)
{
	hetREG1->INTENAS = (1<<(HET_INT_ADDR%32));
	hetRAM1->Instruction[HET_INT_ADDR%32].Control |= 0x01;
}

//=========================================================================
// het interrupt disable function
//=========================================================================
void het_int_disable(void)
{
	hetREG1->INTENAC = (1<<(HET_INT_ADDR%32));
	hetRAM1->Instruction[HET_INT_ADDR%32].Control &= 0xFFFFFFFE;
}
//=========================================================================
// het pwm update function
// Ta,Tb,Tc are variables provide by SVPWM module
// deadzone is time in ns
//=========================================================================
void het_pwm_update(float Ta, float Tb, float Tc, int deadzone)
{
	PWMGEN pwm_drv1	= PWMGEN_DEFAULTS;

	pwm_drv1.deadzone = (deadzone<<(7-HET_LR_REG))*HET_FREQUENCY/(1000*HET_HR);

	pwm_drv1.MfuncC1 = (short)(Ta * 32768);  		// pwm_drv is in Q15
	pwm_drv1.MfuncC2 = (short)(Tb * 32768);
	pwm_drv1.MfuncC3 = (short)(Tc * 32768);
	pwm_drv1.update(&pwm_drv1);

	//Am = HALF_PERIOD - period;
	//Cm = HALF_PERIOD + period;
	hetRAM1->Instruction[PWM_BASE_ADDR].Data	= pwm_drv1.CMPA1;  	// PWM-1A
	hetRAM1->Instruction[PWM_BASE_ADDR+1].Data	= pwm_drv1.CMPD1;

	if(pwm_drv1.CMPA1 < pwm_drv1.deadzone/2)			// PWM-1B
		hetRAM1->Instruction[PWM_BASE_ADDR+2].Data = 0;
	else
		hetRAM1->Instruction[PWM_BASE_ADDR+2].Data = pwm_drv1.CMPA1 - pwm_drv1.deadzone/2;
	if(pwm_drv1.CMPD1+pwm_drv1.deadzone/2 > pwm_drv1.PeriodMax)
		hetRAM1->Instruction[PWM_BASE_ADDR+3].Data = pwm_drv1.PeriodMax + 1;
	else
		hetRAM1->Instruction[PWM_BASE_ADDR+3].Data = pwm_drv1.CMPD1 + pwm_drv1.deadzone/2;

	hetRAM1->Instruction[PWM_BASE_ADDR+4].Data	= pwm_drv1.CMPA3;  	// PWM-2A
	hetRAM1->Instruction[PWM_BASE_ADDR+5].Data	= pwm_drv1.CMPD3;

	if(pwm_drv1.CMPA3 < pwm_drv1.deadzone/2)			// PWM-2B
		hetRAM1->Instruction[PWM_BASE_ADDR+6].Data = 0;
	else
		hetRAM1->Instruction[PWM_BASE_ADDR+6].Data = pwm_drv1.CMPA3 - pwm_drv1.deadzone/2;
	if(pwm_drv1.CMPD3+pwm_drv1.deadzone/2 > pwm_drv1.PeriodMax)
		hetRAM1->Instruction[PWM_BASE_ADDR+7].Data = pwm_drv1.PeriodMax + 1;
	else
		hetRAM1->Instruction[PWM_BASE_ADDR+7].Data = pwm_drv1.CMPD3 + pwm_drv1.deadzone/2;

	hetRAM1->Instruction[PWM_BASE_ADDR+8].Data	= pwm_drv1.CMPA5;  	// PWM-3A
	hetRAM1->Instruction[PWM_BASE_ADDR+9].Data	= pwm_drv1.CMPD5;

	if(pwm_drv1.CMPA5 < pwm_drv1.deadzone/2)			// PWM-3B
		hetRAM1->Instruction[PWM_BASE_ADDR+10].Data = 0;
	else
		hetRAM1->Instruction[PWM_BASE_ADDR+10].Data = pwm_drv1.CMPA5 - pwm_drv1.deadzone/2;
	if(pwm_drv1.CMPD5+pwm_drv1.deadzone/2 > pwm_drv1.PeriodMax)
		hetRAM1->Instruction[PWM_BASE_ADDR+11].Data = pwm_drv1.PeriodMax + 1;
	else
		hetRAM1->Instruction[PWM_BASE_ADDR+11].Data = pwm_drv1.CMPD5 + pwm_drv1.deadzone/2;

	hetRAM1->Instruction[PWM_BASE_ADDR-1].Data = 1<<7;
}



