/* ==================================================================================
File name:		PWM_DRV.H

Originator:		FAE - Texas Instruments

Description: 	Header file containing data type and object definitions and initializers.

Target: 		TMS570R4x family

=====================================================================================
History:
-------------------------------------------------------------------------------------
Date		|	Version	|	Author		|	Description
03.01.2007	|	1.0		|	Gerngross M.|	Original: int (fixed point), pu
----------------------------------------------------------------------------------*/
#ifndef __pwm_drv_H__
#define __pwm_drv_H__
/*-----------------------------------------------------------------------------
Define the structure of the PWMGEN Driver Object
-----------------------------------------------------------------------------*/
typedef struct {
        unsigned short PeriodMax;		// Parameter: PWM -Period in CPU clock cycles (Q0)
        short MfuncC1;        			// Input: PWM 1&2 Duty cycle ratio (Q15)
        short MfuncC2;        			// Input: PWM 3&4 Duty cycle ratio (Q15)
        short MfuncC3;        			// Input: PWM 5&6 Duty cycle ratio (Q15)
		int CMPA1	;		  			// OUTPUT: Update 1
		int CMPA3	;		  			// OUTPUT: Update 3
		int CMPA5	;		  			// OUTPUT: Update 5
		int CMPD1	;		  			// OUTPUT: UpdateD1
		int CMPD3	;		  			// OUTPUT: UpdateD3
		int CMPD5	;		  			// OUTPUT: UpdateD5
		int deadzone;					// Input : deadzone
        void (*update)();     			// Pointer to the update function
        } PWMGEN ;

/*-----------------------------------------------------------------------------
Define a PWMGEN_handle
-----------------------------------------------------------------------------*/
typedef PWMGEN *PWMGEN_handle;

/*------------------------------------------------------------------------------
Default Initializers for the PWMGEN Object
------------------------------------------------------------------------------*/
#define PWMGEN_DEFAULTS		 {(128*(vCNT_MAX + 1))-1,	/*PeriodMax_changed-1*/\
                              0x4000, 				/*MfuncC1*/\
                              0x4000, 				/*MfuncC2*/\
                              0x4000, 				/*MfuncC3*/\
							  0,	  				/*CMPA1*/\
							  0,	  				/*CMPA3*/\
							  0,	  				/*CMPA5*/\
							  0,	  				/*CMPD1*/\
							  0,	  				/*CMPD3*/\
							  0,	  				/*DMPD5*/\
							  100,	  				/*DMPD5*/\
                             (void (*)(unsigned int))PWM_Update \
                             }


/*------------------------------------------------------------------------------
 Prototypes for the functions in PWM_DRV.C
------------------------------------------------------------------------------*/
void PWM_Update(PWMGEN_handle);

#endif  // __pwm_drv_H__

